/*****************************************************************
*  PROGRAM  : cfgSort.cmd                                        *
*  AUTHOR   : N. Morrow - morrownr@netscape.net                  *
*  LANGUAGE : REXX                                               *
*  OS       : eCS v1.1                                           *
*  SYNOPSIS : config.sys sort utility                            *
*  USEAGE   : cfgsort <parameters>, 'cfgsort ?' displays help.   *
*  CREATED  : 2003 May 08                                        *
*  STATUS   : Public Domain                                      *
*  NOTES    : CONFIG.SYS, the primary eCS configuration file,    *
*  appears to be a disorderly combination of various statements. *
*  It is a desirable goal from reviewer, user and tech support   *
*  perspectives to have a config.sys that is very orderly.       *
*                                                                *
*  cfgSort is designed to meet that goal.  cfgSort may be used   *
*  as often as necessary to maintain the organization of the     *
*  config.sys file.  cfgSort runs by default in 'safe' mode      *
*  where it simply outputs a file called config.new in the root  *
*  directory of the boot drive.                                  *
*                                                                *
*  Basic concept of operation:                                   *
*                                                                *
*  cfgSort will sort statements into groups as follows:          *
*                                                                *
*     REMARKS - "REM" statements will be placed in the order     *
*     encountered in the input config.sys.  If one of the        *
*     following words is used in a REM statement then the REM    *
*     statement will remain in the section associated with the   *
*     word:  BASEDEV, DEVINFO, IFS, DEVICE, SET, RUN and CALL.   *
*                                                                *
*     BASE DEVICE DRIVERS - "BASEDEV" statements will be sorted  *
*     by category but individual files within each category will *
*     not be sorted.  The sort order will be as follows:         *
*                                                                *
*     SYS                                                        *
*     BID                                                        *
*     VSD                                                        *
*     TSD                                                        *
*     ADD                                                        *
*     I13                                                        *
*     FLT                                                        *
*     DMD                                                        *
*                                                                *
*     DEVICE INFORMATION - "DEVINFO" statements will be placed   *
*     in the order encountered in the input config.sys.          *
*                                                                *
*     DEVICE DRIVERS - "IFS" statements and "DEVICE" statements  *
*     will be placed in the order encountered in the input       *
*     config.sys.                                                *
*                                                                *
*     PATH STATEMENTS - "PATH" statements will be placed in the  *
*     order encountered in the input config.sys.                 *
*                                                                *
*     CONFIGURATION STATEMENTS - will be alphabetically sorted.  *
*                                                                *
*     SET STATEMENTS - "SET" statements will be placed in the    *
*     order encountered in the input config.sys.                 *
*                                                                *
*     RUN AND CALL STATEMENTS - "RUN" and "CALL" statements      *
*     will be placed in the order encountered in the input       *
*     config.sys.                                                *
*                                                                *
*     -----                                                      *
*                                                                *
*     Change log:                                                *
*                                                                *
*     v3:  Now respects case.  v2 and earlier upper cased all    *
*          characters.  User requested.                          *
*                                                                *
*     v3:  Minor bug fixes and cosmetic changes.                 *
*                                                                *
*     -----                                                      *
*                                                                *
*     Bug reports and recommendations go to:                     *
*                                                                *
*     morrownr@netscape.net                                      *
*                                                                *
*****************************************************************/

/*
   DISCLAIMER:

   THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS "AS IS" AND ANY
   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
   PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
   NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
   OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
   EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

progVer = 'cfgSort v3.0 - 2003.09.17'

parse upper arg arg1

if RxFuncQuery('SysLoadFuncs') then
   do
      call RxFuncAdd 'SysLoadFuncs','RexxUtil','SysLoadFuncs'
      call SysLoadFuncs
   end

if POS('?',arg1) <> 0 then
   do
      call SysCls
      say progVer
      say 
      say 'USAGE: cfgSort <parameters>'
      say 
      say '<parameters> can be any of the following:'
      say 
      say 'R = Replace CONFIG.SYS.  Default behavior is to create CONFIG.NEW'
      say
      say '? = Help'
      say
      say 'cfgSort automatically finds and uses the CONFIG.SYS file located'
      say 'on the eCS boot volume as the input file.  The default behavior'
      say 'is to output a newly processed file called CONFIG.NEW to the root'
      say 'directory of boot volume.  If the "R" parameter is used cfgSort'
      say 'makes a copy of your CONFIG.SYS file in the root directory of the'
      say 'boot drive called CONFIG.BAK.'
      return 1
   end

options exmode

logFile = VALUE('LOGFILES',,'OS2ENVIRONMENT')
if logFile <> '' then logFile = logFile||'\cfgsort.log'

inputFile = SysBootDrive()||"\"||"CONFIG.SYS"
call SysFileTree inputFile, 'fileExist', 'F'
if fileExist.0 = 0 then
   do
      call SysCls
      say progVer
      say 
      say 'ERROR: Cannot find file: '||inputFile
      say 
      return 1
   end

backUpFile = SysBootDrive()||"\"||"CONFIG.BAK"

if arg1 = 'R' then
   do
      outputFile = SysBootDrive()||"\"||"CONFIG.SYS"
      rc = SysFileDelete(backUpFile)
      '@copy 'inputFile backUpFile '> nul'
   end
else
   do
      outputFile = SysBootDrive()||"\"||"CONFIG.NEW"
   end

/* initialize variables */
rem     = 0 /* REMARKS */
basedev = 0 /* BASE DEVICE DRIVERS */
devinfo = 0 /* DEVICE INFORMATION */
device  = 0 /* DEVICE DRIVERS */
path    = 0 /* PATH STATEMENTS */
cfg     = 0 /* CONFIGURATION STATEMENTS */
set     = 0 /* SET STATEMENTS */
exe     = 0 /* RUN AND CALL STATEMENTS */

/* initialize stem variables */
rems.0     = 0 /* REMARKS */
basedevs.0 = 0 /* BASE DEVICE DRIVERS */
devinfos.0 = 0 /* DEVICE INFORMATION */
devices.0  = 0 /* DEVICE DRIVERS */
paths.0    = 0 /* PATH STATEMENTS */
cfgs.0     = 0 /* CONFIGURATION STATEMENTS */
sets.0     = 0 /* SET STATEMENTS */
exes.0     = 0 /* RUN AND CALL STATEMENTS */

main:
do while LINES(inputFile)
   inputLine = LINEIN(inputFile)
   select
      when LENGTH(inputLine)     = 0                then nop
      when SUBSTR(inputLine,1,5) = 'REM ['          then nop
      when SUBSTR(inputLine,1,6) = 'REM -='         then nop
      when SUBSTR(TRANSLATE(inputLine),1,3) = 'REM' then
      select
         when POS('BASEDEV',TRANSLATE(inputLine)) <> 0 then
            do
               basedev          = basedev + 1
               basedevs.0       = basedev
               basedevs.basedev = inputLine
            end
         when POS('DEVINFO',TRANSLATE(inputLine)) <> 0 then
            do
               devinfo          = devinfo + 1
               devinfos.0       = devinfo
               devinfos.devinfo = inputLine
            end
         when POS('IFS',TRANSLATE(inputLine)) <> 0 then
            do
               device         = device + 1
               devices.0      = device
               devices.device = inputLine
            end
         when POS('DEVICE',TRANSLATE(inputLine)) <> 0 then
            do
               device         = device + 1
               devices.0      = device
               devices.device = inputLine
            end
         when POS('SET',TRANSLATE(inputLine)) <> 0 then
            do
               set      = set + 1
               sets.0   = set
               sets.set = inputLine
            end
         when POS('RUN',TRANSLATE(inputLine)) <> 0 then
            do
               exe      = exe + 1
               exes.0   = exe
               exes.exe = inputLine
            end
         when POS('CALL',TRANSLATE(inputLine)) <> 0 then
            do
               exe      = exe + 1
               exes.0   = exe
               exes.exe = inputLine
            end
      otherwise
         do
            rem      = rem + 1
            rems.0   = rem
            rems.rem = inputLine
         end
      end /* end REM processing */

      when SUBSTR(TRANSLATE(inputLine),1,7) = 'BASEDEV' then
         do
            basedev          = basedev + 1
            basedevs.0       = basedev
            basedevs.basedev = inputLine
         end

      when SUBSTR(TRANSLATE(inputLine),1,7) = 'DEVINFO' then
         do
            devinfo          = devinfo + 1
            devinfos.0       = devinfo
            devinfos.devinfo = inputLine
         end

      when SUBSTR(TRANSLATE(inputLine),1,3) = 'IFS' then
         do
            device         = device + 1
            devices.0      = device
            devices.device = inputLine
         end

      when SUBSTR(TRANSLATE(inputLine),1,6) = 'DEVICE' then
         do
            device         = device + 1
            devices.0      = device
            devices.device = inputLine
         end

      when POS('PATH',TRANSLATE(inputLine)) <> 0 then
         do
            path       = path + 1
            paths.0    = path
            paths.path = inputLine
         end

      when SUBSTR(TRANSLATE(inputLine),1,3) = 'SET' then
         do
            set      = set + 1
            sets.0   = set
            sets.set = inputLine
         end

      when SUBSTR(TRANSLATE(inputLine),1,3) = 'RUN' then
         do
            exe      = exe + 1
            exes.0   = exe
            exes.exe = inputLine
         end

      when SUBSTR(TRANSLATE(inputLine),1,4) = 'CALL' then
         do
            exe      = exe + 1
            exes.0   = exe
            exes.exe = inputLine
         end

   otherwise
      do
         cfg      = cfg + 1
         cfgs.0   = cfg
         cfgs.cfg = inputLine
      end
   end  /* select */
end /* main */
call STREAM inputFile,'C','CLOSE'


outputNewFile:
rc = SysFileDelete(outputFile)

call LINEOUT outputFile, 'REM ['inputFile']'
call LINEOUT outputFile, 'REM ['date('O')' 'time()']'
 
call LINEOUT outputFile, ''
call LINEOUT outputFile, 'REM [REMARKS]'
do i = 1 to rems.0
   call LINEOUT outputFile, rems.i
end

call LINEOUT outputFile, ''
call LINEOUT outputFile, 'REM [BASE DEVICE DRIVERS]'
/* sort order */
/*
   SYS
   BID
   VSD
   TSD
   ADD
   I13
   FLT
   DMD
*/
do i = 1 to basedevs.0
   if POS('.SYS',TRANSLATE(basedevs.i)) <> 0 then
      do
         call LINEOUT outputFile, basedevs.i
      end
end
do i = 1 to basedevs.0
   if POS('.BID',TRANSLATE(basedevs.i)) <> 0 then
      do
         call LINEOUT outputFile, basedevs.i
      end
end
do i = 1 to basedevs.0
   if POS('.VSD',TRANSLATE(basedevs.i)) <> 0 then
      do
         call LINEOUT outputFile, basedevs.i
      end
end
do i = 1 to basedevs.0
   if POS('.TSD',TRANSLATE(basedevs.i)) <> 0 then
      do
         call LINEOUT outputFile, basedevs.i
      end
end
do i = 1 to basedevs.0
   if POS('.ADD',TRANSLATE(basedevs.i)) <> 0 then
      do
         call LINEOUT outputFile, basedevs.i
      end
end
do i = 1 to basedevs.0
   if POS('.I13',TRANSLATE(basedevs.i)) <> 0 then
      do
         call LINEOUT outputFile, basedevs.i
      end
end
do i = 1 to basedevs.0
   if POS('.FLT',TRANSLATE(basedevs.i)) <> 0 then
      do
         call LINEOUT outputFile, basedevs.i
      end
end
do i = 1 to basedevs.0
   if POS('.DMD',TRANSLATE(basedevs.i)) <> 0 then
      do
         call LINEOUT outputFile, basedevs.i
      end
end

call LINEOUT outputFile, ''
call LINEOUT outputFile, 'REM [DEVICE INFORMATION]'
do i = 1 to devinfos.0
   call LINEOUT outputFile, devinfos.i
end

call LINEOUT outputFile, ''
call LINEOUT outputFile, 'REM [INSTALLABLE FILE SYSTEMS AND DEVICE DRIVERS]'
do i = 1 to devices.0
   call LINEOUT outputFile, devices.i
end

call LINEOUT outputFile, ''
call LINEOUT outputFile, 'REM [PATH STATEMENTS]'
/* sort disabled
do i = paths.0 to 1 by -1 until flip_flop = 1
   flip_flop = 1
   do j = 2 to i
      m = j - 1
      if paths.m >> paths.j then
      do
         xchg      = paths.m
         paths.m   = paths.j
         paths.j   = xchg
         flip_flop = 0
      end
   end
end
*/
do i = 1 to paths.0
   call LINEOUT outputFile, paths.i
end

call LINEOUT outputFile, ''
call LINEOUT outputFile, 'REM [CONFIGURATION STATEMENTS]'
/* begin sort */
do i = cfgs.0 to 1 by -1 until flip_flop = 1
   flip_flop = 1
   do j = 2 to i
      m = j - 1
      if cfgs.m >> cfgs.j then
      do
         xchg      = cfgs.m
         cfgs.m    = cfgs.j
         cfgs.j    = xchg
         flip_flop = 0
      end
   end
end /* end sort */
do i = 1 to cfgs.0
   call LINEOUT outputFile, cfgs.i
end

call LINEOUT outputFile, ''
call LINEOUT outputFile, 'REM [SET STATEMENTS]'
/* sort disabled
do i = sets.0 to 1 by -1 until flip_flop = 1
   flip_flop = 1
   do j = 2 to i
      m = j - 1
      if sets.m >> sets.j then
      do
         xchg      = sets.m
         sets.m    = sets.j
         sets.j    = xchg
         flip_flop = 0
      end
   end
end
*/
do i = 1 to sets.0
   call LINEOUT outputFile, sets.i
end

call LINEOUT outputFile, ''
call LINEOUT outputFile, 'REM [RUN AND CALL STATEMENTS]'
do i = 1 to exes.0
   call LINEOUT outputFile, exes.i
end

say
say ' ' outputFile 'saved'
say
rc = STREAM(outputFile,'C','CLOSE')
return 0
